//
//  MotorPositionController.h
//  MotorPositionController.h
//
//  Created by Phidgets on 2017-11-19.
//  Copyright (c) 2015 Phidgets. All rights reserved.
//

#import "MotorPositionController.h"
#define SAVE                    0
#define REVERT_SAVE             1
#define REVERT_DEFAULTS         2
#define STALLVELOCITY           7
#define CURRENTLIMIT            8
#define CURRENTREGULATORGAIN    9
#define ENCODERIOMODE           10
struct defaultMotorSettings
{
    double Kp;
    double Ki;
    double Kd;
    double acceleration;
    double velocity;
    double rescaleFactor;
    double deadband;
    double currentLimit;
    double currentRegulatorGain;
    double stallVelocity;
    Phidget_EncoderIOMode iomode;
};

NSArray *saveArray;

@implementation MotorPositionController

- (void)windowWillClose:(NSNotification*)aNotification{
    if(ch){
        //Safely close the example
        Phidget_setOnAttachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnDetachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnErrorHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_close((PhidgetHandle)ch);
        PhidgetMotorPositionController_delete(&ch);
        ch = NULL;
    }
}

struct defaultMotorSettings defaults;
-(void)awakeFromNib{
    PhidgetReturnCode result;
    
    //Manage GUI
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    frame.size.width = [mainWindow minSize].width;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
    [mainWindow setDelegate:self];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    [phidgetInfoBox setHidden:NO];
    
    result = PhidgetMotorPositionController_create(&ch);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    
    result = [self initChannel:(PhidgetHandle)ch];
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    
    result = PhidgetMotorPositionController_setOnPositionChangeHandler(ch, gotPositionChangeData, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    
    result = PhidgetMotorPositionController_setOnDutyCycleUpdateHandler(ch, gotDutyCycleUpdateData, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    
    saveArray = [NSArray arrayWithObjects: @"rescaleFactor", @"kp",@"ki",@"kd",@"acceleration",@"velocityLimit",@"deadband",@"stallVelocity",@"currentLimit", @"currentRegulatorGain", @"encoderIOMode", nil];
    
    [PhidgetInfoBox openCmdLine:(PhidgetHandle)ch];
}

-(PhidgetReturnCode)initChannel:(PhidgetHandle) channel{
    PhidgetReturnCode result;
    
    result = Phidget_setOnAttachHandler(channel, gotAttach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnDetachHandler(channel, gotDetach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnErrorHandler(channel, gotError, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    /*
     * Please review the Phidget22 channel matching documentation for details on the device
     * and class architecture of Phidget22, and how channels are matched to device features.
     */
    
    /*
     * Specifies the serial number of the device to attach to.
     * For VINT devices, this is the hub serial number.
     *
     * The default is any device.
     */
    // Phidget_setDeviceSerialNumber(ch, <YOUR DEVICE SERIAL NUMBER>);
    
    /*
     * For VINT devices, this specifies the port the VINT device must be plugged into.
     *
     * The default is any port.
     */
    // Phidget_setHubPort(ch, 0);
    
    /*
     * Specifies that the channel should only match a VINT hub port.
     * The only valid channel id is 0.
     *
     * The default is 0 (false), meaning VINT hub ports will never match
     */
    // Phidget_setIsHubPortDevice(ch, 1);
    
    /*
     * Specifies which channel to attach to.  It is important that the channel of
     * the device is the same class as the channel that is being opened.
     *
     * The default is any channel.
     */
    // Phidget_setChannel(ch, 0);
    
    /*
     * In order to attach to a network Phidget, the program must connect to a Phidget22 Network Server.
     * In a normal environment this can be done automatically by enabling server discovery, which
     * will cause the client to discovery and connect to available servers.
     *
     * To force the channel to only match a network Phidget, set remote to 1.
     */
    // PhidgetNet_enableServerDiscovery(PHIDGETSERVER_DEVICEREMOTE);
    // Phidget_setIsRemote(ch, 1);
    
    return EPHIDGET_OK;
}


#pragma mark Event callbacks
static void gotAttach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onAttachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}
static void gotDetach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onDetachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}
static void gotError(PhidgetHandle phid, void *context, Phidget_ErrorEventCode errcode, const char *error){
    [(__bridge id)context performSelectorOnMainThread:@selector(errorHandler:)
                                           withObject:[NSArray arrayWithObjects:[NSNumber numberWithInt:errcode], [NSString stringWithUTF8String:error], nil]
                                        waitUntilDone:NO];
}
void gotPositionChangeData(PhidgetMotorPositionControllerHandle phid, void *context, double positionChange){
        [(__bridge id)context performSelectorOnMainThread:@selector(onPositionChangeHandler:)
                                               withObject:[NSNumber numberWithDouble:positionChange]
                                            waitUntilDone:NO];
}
void gotDutyCycleUpdateData(PhidgetMotorPositionControllerHandle phid, void *context, double dutyCycle){
    [(__bridge id)context performSelectorOnMainThread:@selector(onDutyCycleUpdateHandler:)
                                           withObject:[NSNumber numberWithDouble:dutyCycle]
                                        waitUntilDone:NO];
}

#pragma mark Attach, detach, data, and error events
- (void)onAttachHandler{
    PhidgetReturnCode result;
    Phidget_DeviceID deviceID;
    [phidgetInfoBoxView fillPhidgetInfo:(PhidgetHandle)ch];
    //Get device id
    result = Phidget_getDeviceID((PhidgetHandle)ch, &deviceID);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    
    //Grab default values
    result = PhidgetMotorPositionController_getRescaleFactor(ch, &defaults.rescaleFactor);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    result = PhidgetMotorPositionController_getKp(ch, &defaults.Kp);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    result = PhidgetMotorPositionController_getKi(ch, &defaults.Ki);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    result = PhidgetMotorPositionController_getKd(ch, &defaults.Kd);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    result = PhidgetMotorPositionController_getAcceleration(ch, &defaults.acceleration);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    result = PhidgetMotorPositionController_getVelocityLimit(ch, &defaults.velocity);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    result = PhidgetMotorPositionController_getDeadBand(ch, &defaults.deadband);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    
    switch(deviceID){
        case PHIDID_DCC1000:
            result = PhidgetMotorPositionController_getCurrentLimit(ch, &defaults.currentLimit);
            if(result != EPHIDGET_OK){
                [self outputLastError];
            }
            result = PhidgetMotorPositionController_getIOMode(ch, &defaults.iomode);
            if(result != EPHIDGET_OK){
                [self outputLastError];
            }
            result = PhidgetMotorPositionController_getCurrentRegulatorGain(ch, &defaults.currentRegulatorGain);
            if(result != EPHIDGET_OK){
                [self outputLastError];
            }
            break;
        case PHIDID_DCC1001:
            result = PhidgetMotorPositionController_getCurrentLimit(ch, &defaults.currentLimit);
            if(result != EPHIDGET_OK){
                [self outputLastError];
            }
            break;
        case PHIDID_DCC1100:
            result = PhidgetMotorPositionController_getStallVelocity(ch, &defaults.stallVelocity);
            if(result != EPHIDGET_OK){
                [self outputLastError];
            }
        default:
            break;
    }
    
    
    //Set up "other motor settings" box, which is initially hidden
    switch(deviceID){
        case PHIDID_DCC1000:
            [currentLimitBox setHidden:NO];
            [currentRegulatorGainBox setHidden:NO];
            [IOModeBox setHidden:NO];
            [fanModeBox setHidden:NO];
            [encoderIOModeCombo selectItemAtIndex:0];
            [fanModeCombo selectItemAtIndex:2];
            break;
        case PHIDID_DCC1001:
            [currentLimitBox setHidden:NO];
            break;
        case PHIDID_DCC1100:
            [stallVelocityBox setHidden:NO];
        default:
            break;
    }
    
    
    //Try applying saved settings here
    [self applySavedSettings];

    
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow maxSize].height;
    frame.size.width = [mainWindow maxSize].width;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

- (void)onDetachHandler{
    //Taking items out of view
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    [kpText setStringValue:@"0"];
    [kiText setStringValue:@"0"];
    [kdText setStringValue:@"0"];
    [targetPositionText setStringValue:@"0"];
    [accelerationText setStringValue:@"0"];
    [velocityText setStringValue:@"0"];
    [deadbandText setStringValue:@"0"];
    [motorPositionLabel setStringValue:@"0"];
    [motorDutyCycleLabel setStringValue:@"0"];
    [stallVelocityText setStringValue:@"0"];
    [currentLimitText setStringValue:@"0"];
    [currentRegulatorGainText setStringValue:@"0"];
    [encoderIOModeCombo selectItemAtIndex:0];
    [fanModeCombo selectItemAtIndex:2];
    
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    frame.size.width = [mainWindow minSize].width;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

- (void)onPositionChangeHandler:(NSNumber *)position{
    [motorPositionLabel setStringValue:[NSString stringWithFormat:@"%0.3f", position.doubleValue]];
}

- (void)onDutyCycleUpdateHandler:(NSNumber *)dutyCycle{
    [motorDutyCycleLabel setStringValue:[NSString stringWithFormat:@"%0.3f", dutyCycle.doubleValue]];
}

static int errorCounter = 0;
-(void) outputError:(const char *)errorString{
    errorCounter++;
    NSAttributedString *outputString = [[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@\n",[NSString stringWithUTF8String:errorString]]
    	attributes:@{NSForegroundColorAttributeName: NSColor.controlTextColor}];
    [[errorEventLog textStorage] beginEditing];
    [[errorEventLog textStorage] appendAttributedString:outputString];
    [[errorEventLog textStorage] endEditing];
    
    [errorEventLogCounter setIntValue:errorCounter];
    if(![errorEventLogWindow isVisible])
        [errorEventLogWindow setIsVisible:YES];
}

-(void)errorHandler:(NSArray *)errorEventData{
    const char* errorString = [[errorEventData objectAtIndex:1] UTF8String];
    [self outputError:errorString];
    if([[errorEventData objectAtIndex:0] intValue] == EEPHIDGET_MOTORSTALL){
        [engageMotorButton setTitle:@"Engage Motor"];
    }
}

-(void)outputLastError {
	const char *errorString;
	char *errorDetailString = NULL;
    size_t errorDetailStringLen = 0;
    PhidgetReturnCode lastErr;
	
    if (!Phidget_getLastError(&lastErr, &errorString, NULL, &errorDetailStringLen)) {
		errorDetailString = malloc(errorDetailStringLen);
		Phidget_getLastError(&lastErr, &errorString, errorDetailString, &errorDetailStringLen);
    	[self outputError:errorDetailString];
    	free(errorDetailString);
    }
}

- (IBAction)clearErrorLog:(id)sender{
    [[errorEventLog textStorage] setAttributedString:[[NSAttributedString alloc] initWithString:@""]];
    [errorEventLogCounter setIntValue:0];
    errorCounter = 0;
}

-(double)updateLimit: (double)n{
    n = fabs(n);
    if(n != 0 && n > 1){
        return pow(10,floor(log10(n)))/2;
    }
    else{
        return 0.5;
    }
}

-(void)applySavedSettings{
    Phidget_DeviceID deviceID;
    PhidgetReturnCode result;
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    result = Phidget_getDeviceID((PhidgetHandle)ch, &deviceID);
    
    if([userDefaults objectForKey:@"increment"]){
        double increment;
        increment = [userDefaults doubleForKey:@"increment"];
        targetPositionStepper.increment = increment;
        [incrementPositionText setStringValue:[NSString stringWithFormat:@"%.2f",increment]];
    }
    
    for(int i = 0; i < 11; i++){
        if([userDefaults objectForKey:saveArray[i]]){
            double savedValue = [userDefaults doubleForKey:saveArray[i]];
            if(i == 0)
                result = PhidgetMotorPositionController_setRescaleFactor(ch, savedValue);
            else if(i == 1)
                result = PhidgetMotorPositionController_setKp(ch, savedValue);
            else if(i == 2)
                result = PhidgetMotorPositionController_setKi(ch, savedValue);
            else if(i == 3)
                result = PhidgetMotorPositionController_setKd(ch, savedValue);
            else if(i == 4)
                result = PhidgetMotorPositionController_setAcceleration(ch, savedValue);
            else if(i == 5)
                result = PhidgetMotorPositionController_setVelocityLimit(ch, savedValue);
            else if(i == 6)
                result = PhidgetMotorPositionController_setDeadBand(ch, savedValue);
            else if(i == 7 && deviceID == PHIDID_DCC1100)
                result = PhidgetMotorPositionController_setStallVelocity(ch, savedValue);
            else if(i == 8 && (deviceID == PHIDID_DCC1000 || deviceID == PHIDID_DCC1001))
                result = PhidgetMotorPositionController_setCurrentLimit(ch, savedValue);
            else if(i == 9 && deviceID == PHIDID_DCC1000)
                result = PhidgetMotorPositionController_setCurrentRegulatorGain(ch, savedValue);
            else if(i == 10 && deviceID == PHIDID_DCC1000)
                result = PhidgetMotorPositionController_setIOMode(ch, savedValue+1);
            
            if(result != EPHIDGET_OK){
                [self outputLastError];
            }
        }
    }
    
    [self updateGUI];
}

-(void)updateRescaleProperties{
    double minAcceleration, maxAcceleration, acceleration;
    double minVelocityLimit, maxVelocityLimit, velocityLimit;
    double deadband, maxPosition, minPosition;
    double minStallVelocity, maxStallVelocity, stallVelocity;
    PhidgetReturnCode result;
    Phidget_DeviceID deviceID;
    
    result = Phidget_getDeviceID((PhidgetHandle)ch, &deviceID);
    
    result = PhidgetMotorPositionController_getMinAcceleration(ch, &minAcceleration);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    result = PhidgetMotorPositionController_getMaxAcceleration(ch, &maxAcceleration);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    result = PhidgetMotorPositionController_getAcceleration(ch, &acceleration);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    result = PhidgetMotorPositionController_getMinVelocityLimit(ch, &minVelocityLimit);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    result = PhidgetMotorPositionController_getMaxVelocityLimit(ch, &maxVelocityLimit);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    result = PhidgetMotorPositionController_getVelocityLimit(ch, &velocityLimit);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    result = PhidgetMotorPositionController_getDeadBand(ch, &deadband);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    result = PhidgetMotorPositionController_getMinPosition(ch, &minPosition);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    result = PhidgetMotorPositionController_getMaxPosition(ch, &maxPosition);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    if(deviceID == PHIDID_DCC1100){
        result = PhidgetMotorPositionController_getMinStallVelocity(ch, &minStallVelocity);
        if(result != EPHIDGET_OK){
            [self outputLastError];
        }
        result = PhidgetMotorPositionController_getMaxStallVelocity(ch, &maxStallVelocity);
        if(result != EPHIDGET_OK){
            [self outputLastError];
        }
        result = PhidgetMotorPositionController_getStallVelocity(ch, &stallVelocity);
        if(result != EPHIDGET_OK){
            [self outputLastError];
        }
        [stallVelocityStepper setIncrement:[self updateLimit:stallVelocity]];
        [stallVelocityStepper setMinValue:minStallVelocity];
        [stallVelocityStepper setMaxValue:maxStallVelocity];
        [stallVelocityStepper setDoubleValue:stallVelocity];
        [stallVelocityText setStringValue:[NSString stringWithFormat:@"%.2f",stallVelocity]];
    }
    
    [accelerationText setStringValue:[NSString stringWithFormat:@"%.2f", acceleration]];
    [velocityText setStringValue:[NSString stringWithFormat:@"%.2f", velocityLimit]];
    [accelerationStepper setIncrement:[self updateLimit:acceleration]];
    [accelerationStepper setMinValue:minAcceleration];
    [accelerationStepper setMaxValue:maxAcceleration];
    [accelerationStepper setDoubleValue:acceleration];
    
    [velocityStepper setIncrement:[self updateLimit:velocityLimit]];
    [velocityStepper setMinValue:minVelocityLimit];
    [velocityStepper setMaxValue:maxVelocityLimit];
    [velocityStepper setDoubleValue:velocityLimit];
    
    targetPositionStepper.minValue = minPosition;
    targetPositionStepper.maxValue = maxPosition;
    [deadbandText setStringValue:[NSString stringWithFormat:@"%f",deadband]];
    
}

-(void)updateGUI{
    double kp, ki, kd, rescale,currentLimit, minCurrentLimit, maxCurrentLimit, currentRegulatorGain, minCurrentRegulatorGain, maxCurrentRegulatorGain,stallVelocity,minStallVelocity,maxStallVelocity;
    PhidgetReturnCode result;
    Phidget_DeviceID deviceID;
    Phidget_EncoderIOMode ioMode;
    
    result = Phidget_getDeviceID((PhidgetHandle)ch, &deviceID);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    result = PhidgetMotorPositionController_getKp(ch, &kp);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    result = PhidgetMotorPositionController_getKi(ch, &ki);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    result = PhidgetMotorPositionController_getKd(ch, &kd);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    
    result = PhidgetMotorPositionController_getRescaleFactor(ch, &rescale);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }

    if(deviceID == PHIDID_DCC1000 || deviceID == PHIDID_DCC1001){
        result = PhidgetMotorPositionController_getCurrentLimit(ch, &currentLimit);
        if(result != EPHIDGET_OK){
            [self outputLastError];
        }
        result = PhidgetMotorPositionController_getMinCurrentLimit(ch, &minCurrentLimit);
        if(result != EPHIDGET_OK){
            [self outputLastError];
        }
        result = PhidgetMotorPositionController_getMaxCurrentLimit(ch, &maxCurrentLimit);
        if(result != EPHIDGET_OK){
            [self outputLastError];
        }
        [currentLimitText setStringValue:[NSString stringWithFormat:@"%.2f",currentLimit]];
        [currentLimitStepper setDoubleValue:currentLimit];
        currentLimitStepper.minValue = minCurrentLimit;
        currentLimitStepper.maxValue = maxCurrentLimit;
    }
    if(deviceID == PHIDID_DCC1000){
        result = PhidgetMotorPositionController_getIOMode(ch, &ioMode);
        if(result != EPHIDGET_OK){
            [self outputLastError];
        }
        result = PhidgetMotorPositionController_getCurrentRegulatorGain(ch, &currentRegulatorGain);
        if(result != EPHIDGET_OK){
            [self outputLastError];
        }
        result = PhidgetMotorPositionController_getMinCurrentRegulatorGain(ch, &minCurrentRegulatorGain);
        if(result != EPHIDGET_OK){
            [self outputLastError];
        }
        result = PhidgetMotorPositionController_getMaxCurrentRegulatorGain(ch, &maxCurrentRegulatorGain);
        if(result != EPHIDGET_OK){
            [self outputLastError];
        }
        [currentRegulatorGainText setStringValue:[NSString stringWithFormat:@"%.2f",currentRegulatorGain]];
        [currentRegulatorGainStepper setDoubleValue:currentRegulatorGain];
        currentRegulatorGainStepper.minValue = minCurrentRegulatorGain;
        currentRegulatorGainStepper.maxValue = maxCurrentRegulatorGain;
        [encoderIOModeCombo selectItemAtIndex:(int)ioMode - 1];
    }
    if(deviceID == PHIDID_DCC1100){
        result = PhidgetMotorPositionController_getStallVelocity(ch, &stallVelocity);
        if(result != EPHIDGET_OK){
            [self outputLastError];
        }
        result = PhidgetMotorPositionController_getMaxStallVelocity(ch, &maxStallVelocity);
        if(result != EPHIDGET_OK){
            [self outputLastError];
        }
        result = PhidgetMotorPositionController_getMinStallVelocity(ch, &minStallVelocity);
        if(result != EPHIDGET_OK){
            [self outputLastError];
        }
        [stallVelocityText setStringValue:[NSString stringWithFormat:@"%.2f",stallVelocity]];
        [stallVelocityStepper setDoubleValue:stallVelocity];
        stallVelocityStepper.minValue = minStallVelocity;
        stallVelocityStepper.maxValue = maxStallVelocity;
    }
    
    [kpText setStringValue:[NSString stringWithFormat:@"%.1f",kp]];
    [kiText setStringValue:[NSString stringWithFormat:@"%.1f",ki]];
    [kdText setStringValue:[NSString stringWithFormat:@"%.1f",kd]];
    [rescaleText setStringValue:[NSString stringWithFormat:@"%f",rescale]];
    
    [self updateRescaleProperties];
    
    //Set stepper limits
    kpStepper.increment = [self updateLimit:kp];
    kiStepper.increment = [self updateLimit:ki];
    kdStepper.increment = [self updateLimit:kd];
    [kpStepper setDoubleValue:kp];
    [kiStepper setDoubleValue:ki];
    [kdStepper setDoubleValue:kd];

}

#pragma mark GUI controls
//Kp, Ki, Kd
- (IBAction)setKp:(id)sender {
    PhidgetReturnCode result;
    result = PhidgetMotorPositionController_setKp(ch, kpText.stringValue.doubleValue);
    if(result != EPHIDGET_OK){
        [self outputLastError];
        return;
    }
    [kpStepper setDoubleValue:kpText.stringValue.doubleValue];
    kpStepper.increment = [self updateLimit:kpStepper.doubleValue];
}
- (IBAction)kpIncrement:(id)sender {
    PhidgetReturnCode result;
    result = PhidgetMotorPositionController_setKp(ch, kpStepper.doubleValue);
    if(result != EPHIDGET_OK){
        [self outputLastError];
        return;
    }
    [kpText setStringValue:[NSString stringWithFormat:@"%.1f",kpStepper.doubleValue]];
    kpStepper.increment = [self updateLimit:kpStepper.doubleValue];
}

- (IBAction)setKi:(id)sender {
    PhidgetReturnCode result;
    result = PhidgetMotorPositionController_setKi(ch, kiText.stringValue.doubleValue);
    if(result != EPHIDGET_OK){
        [self outputLastError];
        return;
    }
    [kiStepper setDoubleValue:kiText.stringValue.doubleValue];
    kiStepper.increment = [self updateLimit:kiStepper.doubleValue];
}
- (IBAction)kiIncrement:(id)sender {
    PhidgetReturnCode result;
    result = PhidgetMotorPositionController_setKi(ch, kiStepper.doubleValue);
    if(result != EPHIDGET_OK){
        [self outputLastError];
        return;
    }
    [kiText setStringValue:[NSString stringWithFormat:@"%.1f",kiStepper.doubleValue]];
    kiStepper.increment = [self updateLimit:kiStepper.doubleValue];
}

- (IBAction)setKd:(id)sender {
    PhidgetReturnCode result;
    result = PhidgetMotorPositionController_setKd(ch, kdText.stringValue.doubleValue);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    [kdStepper setDoubleValue:kdText.stringValue.doubleValue];
    kdStepper.increment = [self updateLimit:kdStepper.doubleValue];
}
- (IBAction)kdIncrement:(id)sender {
    PhidgetReturnCode result;
    result = PhidgetMotorPositionController_setKd(ch, kdStepper.doubleValue);
    if(result != EPHIDGET_OK){
        [self outputLastError];
        return;
    }
    [kdText setStringValue:[NSString stringWithFormat:@"%.1f",kdStepper.doubleValue]];
    kdStepper.increment = [self updateLimit:kdStepper.doubleValue];
}

//Acceleration and Velocity Limit
- (IBAction)setAcceleration:(id)sender {
    PhidgetReturnCode result;
    result = PhidgetMotorPositionController_setAcceleration(ch, accelerationText.stringValue.doubleValue);
    if(result != EPHIDGET_OK){
        [self outputLastError];
        return;
    }
    [accelerationStepper setDoubleValue:accelerationText.stringValue.doubleValue];
    accelerationStepper.increment = [self updateLimit:accelerationStepper.doubleValue];
}
- (IBAction)accelerationIncrement:(id)sender {
    PhidgetReturnCode result;
    result = PhidgetMotorPositionController_setAcceleration(ch, accelerationStepper.doubleValue);
    if(result != EPHIDGET_OK){
        [self outputLastError];
        return;
    }
    [accelerationText setStringValue:[NSString stringWithFormat:@"%.2f",accelerationStepper.doubleValue]];
    accelerationStepper.increment = [self updateLimit:accelerationStepper.doubleValue];
}

- (IBAction)setVelocityLimit:(id)sender {
    PhidgetReturnCode result;
    result = PhidgetMotorPositionController_setVelocityLimit(ch, velocityText.stringValue.doubleValue);
    if(result != EPHIDGET_OK){
        [self outputLastError];
        return;
    }
    [velocityStepper setDoubleValue:velocityText.stringValue.doubleValue];
    velocityStepper.increment = [self updateLimit:velocityStepper.doubleValue];
}
- (IBAction)velocityIncrement:(id)sender {
    PhidgetReturnCode result;
    result = PhidgetMotorPositionController_setVelocityLimit(ch, velocityStepper.doubleValue);
    if(result != EPHIDGET_OK){
        [self outputLastError];
        return;
    }
    [velocityText setStringValue:[NSString stringWithFormat:@"%.2f",velocityStepper.doubleValue]];
    velocityStepper.increment = [self updateLimit:velocityStepper.doubleValue];
}

//Rescale Factor and Deadband
- (IBAction)setRescaleFactor:(id)sender {
    PhidgetReturnCode result;
    result = PhidgetMotorPositionController_setRescaleFactor(ch, rescaleText.stringValue.doubleValue);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    [self updateRescaleProperties];
}
- (IBAction)setDeadband:(id)sender {
    PhidgetReturnCode result;
    result = PhidgetMotorPositionController_setDeadBand(ch, deadbandText.stringValue.doubleValue);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
}

//Target position
- (IBAction)setTargetPosition:(id)sender {
    PhidgetReturnCode result;
    result = PhidgetMotorPositionController_setTargetPosition(ch, targetPositionText.stringValue.doubleValue);
    if(result != EPHIDGET_OK){
        [self outputLastError];
        return;
    }
    [targetPositionStepper setDoubleValue:targetPositionText.stringValue.doubleValue];
}
- (IBAction)targetPositionIncrement:(id)sender {
    PhidgetReturnCode result;
    result = PhidgetMotorPositionController_setTargetPosition(ch, [targetPositionStepper doubleValue]);
    if(result != EPHIDGET_OK){
        [self outputLastError];
        return;
    }
    [targetPositionText setStringValue:[NSString stringWithFormat:@"%.2f",[targetPositionStepper doubleValue]]];
}

- (IBAction)setIncrement:(id)sender {
    targetPositionStepper.increment = incrementPositionText.stringValue.doubleValue;
    [[NSUserDefaults standardUserDefaults] setDouble:incrementPositionText.stringValue.doubleValue forKey:@"increment"];
}

- (IBAction)engageMotor:(id)sender {
    PhidgetReturnCode result;
    int engaged;
    result = PhidgetMotorPositionController_getEngaged(ch, &engaged);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    if(engaged){
        result = PhidgetMotorPositionController_setEngaged(ch, 0);
        if(result != EPHIDGET_OK){
            [self outputLastError];
        }
        [engageMotorButton setTitle:@"Engage Motor"];
    }
    else{
        result = PhidgetMotorPositionController_setEngaged(ch, 1);
        if(result != EPHIDGET_OK){
            [self outputLastError];
        }
        [engageMotorButton setTitle:@"Disengage Motor"];
    }
}

//Other Motor Settings
- (IBAction)setCurrentLimit:(id)sender {
    PhidgetReturnCode result;
    result = PhidgetMotorPositionController_setCurrentLimit(ch, currentLimitText.stringValue.doubleValue);
    if(result != EPHIDGET_OK){
        [self outputLastError];
        return;
    }
    [currentLimitStepper setDoubleValue:currentLimitText.stringValue.doubleValue];
    currentLimitStepper.increment = [self updateLimit:currentLimitStepper.doubleValue];
}
- (IBAction)currentLimitIncrement:(id)sender {
    PhidgetReturnCode result;
    result = PhidgetMotorPositionController_setCurrentLimit(ch, currentLimitStepper.doubleValue);
    if(result != EPHIDGET_OK){
        [self outputLastError];
        return;
    }
    [currentLimitText setStringValue:[NSString stringWithFormat:@"%.2f",currentLimitStepper.doubleValue]];
    currentLimitStepper.increment = [self updateLimit:currentLimitStepper.doubleValue];
}
- (IBAction)setCurrentRegulatorGain:(id)sender {
    PhidgetReturnCode result;
    result = PhidgetMotorPositionController_setCurrentRegulatorGain(ch, currentRegulatorGainText.stringValue.doubleValue);
    if(result != EPHIDGET_OK){
        [self outputLastError];
        return;
    }
    [currentRegulatorGainStepper setDoubleValue:currentRegulatorGainText.stringValue.doubleValue];
    currentRegulatorGainStepper.increment = [self updateLimit:currentRegulatorGainStepper.doubleValue];
}
- (IBAction)currentRegulatorGainIncrement:(id)sender {
    PhidgetReturnCode result;
    result = PhidgetMotorPositionController_setCurrentRegulatorGain(ch, currentRegulatorGainStepper.doubleValue);
    if(result != EPHIDGET_OK){
        [self outputLastError];
        return;
    }
    [currentRegulatorGainText setStringValue:[NSString stringWithFormat:@"%.2f",currentRegulatorGainStepper.doubleValue]];
    currentRegulatorGainStepper.increment = [self updateLimit:currentRegulatorGainStepper.doubleValue];
}
- (IBAction)setStallVelocity:(id)sender {
    PhidgetReturnCode result;
    result = PhidgetMotorPositionController_setStallVelocity(ch, stallVelocityText.stringValue.doubleValue);
    if(result != EPHIDGET_OK){
        [self outputLastError];
        return;
    }
    [stallVelocityStepper setDoubleValue:stallVelocityText.stringValue.doubleValue];
    stallVelocityStepper.increment = [self updateLimit:stallVelocityStepper.doubleValue];
}
- (IBAction)stallVelocityIncrement:(id)sender {
    PhidgetReturnCode result;
    result = PhidgetMotorPositionController_setStallVelocity(ch, stallVelocityStepper.doubleValue);
    if(result != EPHIDGET_OK){
        [self outputLastError];
        return;
    }
    [stallVelocityText setStringValue:[NSString stringWithFormat:@"%.2f",stallVelocityStepper.doubleValue]];
    stallVelocityStepper.increment = [self updateLimit:stallVelocityStepper.doubleValue];
}
- (IBAction)setIOMode:(id)sender {
    PhidgetReturnCode result;
    if([encoderIOModeCombo indexOfSelectedItem] == 0)
        result = PhidgetMotorPositionController_setIOMode(ch, ENCODER_IO_MODE_PUSH_PULL);
    else if([encoderIOModeCombo indexOfSelectedItem] == 1)
        result = PhidgetMotorPositionController_setIOMode(ch, ENCODER_IO_MODE_LINE_DRIVER_2K2);
    else if([encoderIOModeCombo indexOfSelectedItem] == 2)
        result = PhidgetMotorPositionController_setIOMode(ch, ENCODER_IO_MODE_LINE_DRIVER_10K);
    else if([encoderIOModeCombo indexOfSelectedItem] == 3)
        result = PhidgetMotorPositionController_setIOMode(ch, ENCODER_IO_MODE_OPEN_COLLECTOR_2K2);
    else if([encoderIOModeCombo indexOfSelectedItem] == 4)
        result = PhidgetMotorPositionController_setIOMode(ch, ENCODER_IO_MODE_OPEN_COLLECTOR_10K);
	else
		result = EPHIDGET_UNEXPECTED;
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
}
- (IBAction)setFanMode:(id)sender {
    PhidgetReturnCode result;
    if([fanModeCombo indexOfSelectedItem] == 0)
        result = PhidgetMotorPositionController_setFanMode(ch, FAN_MODE_ON);
    else if([fanModeCombo indexOfSelectedItem] == 1)
        result = PhidgetMotorPositionController_setFanMode(ch, FAN_MODE_OFF);
    else if([fanModeCombo indexOfSelectedItem] == 2)
        result = PhidgetMotorPositionController_setFanMode(ch, FAN_MODE_AUTO);
	else
		result = EPHIDGET_UNEXPECTED;
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
}
- (IBAction)showOtherMotorSettings:(id)sender {
    NSButton *check = (NSButton *)sender;
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    if(check.state){
        frame.size.height = [mainWindow maxSize].height;
    }
    else{
        frame.size.height = [mainWindow maxSize].height - otherMotorSettingsBox.frame.size.height;
    }
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}
- (IBAction)zeroPosition:(id)sender {
    PhidgetReturnCode result;
    double position;
    result = PhidgetMotorPositionController_getPosition(ch, &position);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    result = PhidgetMotorPositionController_addPositionOffset(ch, -1 * position);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
}

//Settings
-(void)showHideSettings{
    Phidget_DeviceID deviceID;
    Phidget_getDeviceID((PhidgetHandle)ch,&deviceID);
    if(deviceID == PHIDID_DCC1000){
        [settingsMatrix cellWithTag:STALLVELOCITY].enabled = NO;
    }
    else if(deviceID == PHIDID_DCC1001){
        [settingsMatrix cellWithTag:STALLVELOCITY].enabled = NO;
        [settingsMatrix cellWithTag:CURRENTREGULATORGAIN].enabled = NO;
        [settingsMatrix cellWithTag:ENCODERIOMODE].enabled = NO;
    }
    else if(deviceID == PHIDID_DCC1100){
        [settingsMatrix cellWithTag:CURRENTLIMIT].enabled = NO;
        [settingsMatrix cellWithTag:CURRENTREGULATORGAIN].enabled = NO;
        [settingsMatrix cellWithTag:ENCODERIOMODE].enabled = NO;
    }

}
int saveState = SAVE;
- (IBAction)saveSettings:(id)sender {
    if(![settingsWindow isVisible]){
        [saveButton setTitle:@"Save"];
        [settingsWindow setIsVisible:YES];
        saveState = SAVE;
        [self showHideSettings];
    }
}
- (IBAction)revertToSavedSettings:(id)sender {
    if(![settingsWindow isVisible]){
        [saveButton setTitle:@"Revert to Saved"];
        [settingsWindow setIsVisible:YES];
        saveState = REVERT_SAVE;
        [self showHideSettings];
    }
}
- (IBAction)revertToDefaultSettings:(id)sender {
    if(![settingsWindow isVisible]){
        [saveButton setTitle:@"Revert to Default"];
        [settingsWindow setIsVisible:YES];
        saveState = REVERT_DEFAULTS;
        [self showHideSettings];
    }
}
- (IBAction)selectAllSettings:(id)sender {
    for (NSCell *cell in [settingsMatrix cells])
    {
        if(cell.enabled)
            cell.state = 1;
    }
}

- (IBAction)deselectAllSettings:(id)sender {
    for (NSCell *cell in [settingsMatrix cells])
    {
        cell.state = 0;
    }
}
- (IBAction)enterSettingsSelection:(id)sender {
    NSUserDefaults *userDefaults;
    userDefaults = [NSUserDefaults standardUserDefaults];
    
    if(saveState == SAVE){
        double valueArray[11];
        valueArray[0] = rescaleText.stringValue.doubleValue;
        valueArray[1] = kpText.stringValue.doubleValue;
        valueArray[2] = kiText.stringValue.doubleValue;
        valueArray[3] = kdText.stringValue.doubleValue;
        valueArray[4] = accelerationText.stringValue.doubleValue;
        valueArray[5] = velocityText.stringValue.doubleValue;
        valueArray[6] = deadbandText.stringValue.doubleValue;
        valueArray[7] = stallVelocityText.stringValue.doubleValue;
        valueArray[8] = currentLimitText.stringValue.doubleValue;
        valueArray[9] = currentRegulatorGainText.stringValue.doubleValue;
        valueArray[10] = encoderIOModeCombo.indexOfSelectedItem;
        for (NSCell *cell in [settingsMatrix cells])
        {
            if(cell.state){
                [userDefaults setDouble:valueArray[cell.tag] forKey:saveArray[cell.tag]];
            }
        }
        [settingsWindow setIsVisible:NO];
        NSAlert *alert = [NSAlert alertWithMessageText:@"Saved" defaultButton:@"OK" alternateButton:nil otherButton:nil informativeTextWithFormat:@"Settings have been saved!"];
        [alert runModal];
    }
    else if (saveState == REVERT_DEFAULTS){
        for (NSCell *cell in [settingsMatrix cells])
        {
            if(cell.state){
                if(cell.tag == 0)
                    PhidgetMotorPositionController_setRescaleFactor(ch, defaults.rescaleFactor);
                else if (cell.tag == 1)
                    PhidgetMotorPositionController_setKp(ch, defaults.Kp);
                else if (cell.tag == 2)
                    PhidgetMotorPositionController_setKi(ch, defaults.Ki);
                else if (cell.tag == 3)
                    PhidgetMotorPositionController_setKd(ch, defaults.Kd);
                else if (cell.tag == 4)
                    PhidgetMotorPositionController_setAcceleration(ch, defaults.acceleration);
                else if (cell.tag == 5)
                    PhidgetMotorPositionController_setVelocityLimit(ch, defaults.velocity);
                else if (cell.tag == 6)
                    PhidgetMotorPositionController_setDeadBand(ch, defaults.deadband);
                else if (cell.tag == 7)
                    PhidgetMotorPositionController_setStallVelocity(ch, defaults.stallVelocity);
                else if (cell.tag == 8)
                    PhidgetMotorPositionController_setCurrentLimit(ch, defaults.currentLimit);
                else if (cell.tag == 9)
                    PhidgetMotorPositionController_setCurrentRegulatorGain(ch, defaults.currentRegulatorGain);
                else if (cell.tag == 10)
                    PhidgetMotorPositionController_setIOMode(ch, ENCODER_IO_MODE_PUSH_PULL);
            }
        }
        [self updateGUI];
        [settingsWindow setIsVisible:NO];
        NSAlert *alert = [NSAlert alertWithMessageText:@"Reverted" defaultButton:@"OK" alternateButton:nil otherButton:nil informativeTextWithFormat:@"Settings have been reverted to defaults!"];
        [alert runModal];
    }
    else if (saveState == REVERT_SAVE){
        [self applySavedSettings];
        [settingsWindow setIsVisible:NO];
        NSAlert *alert = [NSAlert alertWithMessageText:@"Reverted" defaultButton:@"OK" alternateButton:nil otherButton:nil informativeTextWithFormat:@"Settings have been reverted to the saved values!"];
        [alert runModal];
    }
    
}


@end
